<?php
require_once ('op5/mayi.php');

class op5license_Constraints implements op5MayI_Constraints {
	protected $license_backend = false;

	public function __construct() {
		$this->license_backend = op5objstore::instance()->obj_instance_callback('op5license', function () {
			return new op5license();
		});
	}

	/**
	 * Execute a action
	 *
	 * @param string $action
	 *        	name of the action, as "path.to.resource:action"
	 * @param array $env
	 *        	environment variables for the constraints
	 * @param array $messages
	 *        	referenced array to add messages to
	 * @param op5MayIMetric[] $metrics
	 *        	referenced array to add performance data to
	 */
	public function run($action, $env, &$messages, &$metrics) {
		list($resource, $method) = array_map(function($str) { return explode('.', $str); }, explode(':', $action) );

		// There are actions that the license cannot block by its own
		// means, see comments below for which ones. Instead of just
		// returning early though, we should add the messages from the
		// licence, so we need to actually run the license check.
		$force_successful_result = false;

		if($resource[0] == 'ninja') {
			if(isset($resource[1]) && $resource[1] == 'session') {
				// we should handle this case within backbox,
				// not within license, but it needs therefore
				// to be handled from the license
			} elseif($action === 'ninja.auth:login') {
				// the license must be able to stop login
				// attempts
			} else {
				// ninja resource namespace is mostly
				// meta-methods in ninja, which shouldn't be
				// controlled by license
				$force_successful_result = true;
			}
		} elseif($resource[0] == 'monitor' && $resource[1] == 'license') {
			// the license module cannot claim ownership of the
			// answer to the question "can a user create a
			// license?"
			$force_successful_result = true;
		}

		$raw_metrics = array();
		$result = $this->license_backend->run($action, $env, $messages, $raw_metrics);
		foreach($raw_metrics as $name => $values) {
			list($value, $min, $max) = $values;
			$metrics[$name] = new op5MayIMetric($value, $min, $max);
		}

		if($force_successful_result) {
			return true;
		}
		return $result;
	}
}

class ninja_license {
	/**
	 * Create hooks and initialize libraries
	 */
	public function __construct() {
		Event::add('ninja.get_branding', function() {
			$data = &Event::$data['branding'];
			$data['image'] = ninja::add_path('brands/monitor.png', false, true);
		});
		Event::add('ninja.show_login', array ($this,'show_login'));
		Event::add('system.pre_controller', array ($this,'cleanup_session'));

		$mayi = op5MayI::instance();
		$mayi->act_upon(new op5license_Constraints());
	}

	/**
	 * Test if we are allowed to see ninja at all, or see the "blaffa"
	 */
	private function is_licensed() {
		return op5mayi::instance()->run('ninja.session:create.blaffa');
	}

	/**
	 * Hook to show "license has expired" text on login page if system is
	 * unlicensed.
	 */
	public function show_login() {
		if ($this->is_licensed())
			return;

		// Figure out license type and bounce URL.
		$license = op5objstore::instance()->obj_instance_callback('op5license', function () {
			return new op5license();
		});
		$metadata = $license->get_metadata();
		$lictype =
			isset($metadata['branding']) ? $metadata['branding'] : 'unknown';

		$licbounceurl =
			"https://www.op5.com/op5-monitor-license-bounce?lic=" .
			"$lictype&info=expired";

		$message = new View('license_expired', array(
			'license_name'       => license::get_display_name($lictype),
			'license_bounce_url' => $licbounceurl
		));

		// Add error message to template.
		$controller = Event::$data;
		$controller->template->content->message = new ErrorNotice_Model($message->render());
	}
	/**
	 * Destroy session if unlicensed.
	 * That means, we shouldn't be able to be logged in at all to ninja
	 */
	public function cleanup_session() {
		if (!$this->is_licensed()) {
			Session::instance()->destroy();

			Auth::disable_auth_subsystem();
		}
	}
}

new ninja_license();
